Package: ggsc
Title: Visualizing Single Cell and Spatial Transcriptomics
Version: 1.10.1
Authors@R: c(
       person("Guangchuang", "Yu", 
              email = "guangchuangyu@gmail.com", 
              role = c("aut", "cre", "cph"), 
              comment = c(ORCID = "0000-0002-6485-8781")
       ),
       person("Shuangbin", "Xu",
              email = "xshuangbin@163.com",      
              role = "aut", 
              comment = c(ORCID="0000-0003-3513-5362")
       ),
       person("Noriaki", "Sato", 
              email = "nori@hgc.jp", 
              role = "ctb")
  )
Description: Useful functions to visualize single cell and spatial
        data. It supports visualizing 'Seurat', 'SingleCellExperiment'
        and 'SpatialExperiment' objects through grammar of graphics
        syntax implemented in 'ggplot2'.
Depends: R (>= 4.1.0)
Imports: Rcpp, RcppParallel, cli, dplyr, ggfun (>= 0.1.5), ggplot2,
        grDevices, grid, methods, rlang, scattermore, stats, Seurat,
        SingleCellExperiment, SummarizedExperiment, tidydr, tidyr,
        tibble, utils, RColorBrewer, yulab.utils, scales
Suggests: aplot, BiocParallel, forcats, ggforce, ggnewscale, igraph,
        knitr, ks, Matrix, prettydoc, rmarkdown, scran, scater,
        scatterpie (>= 0.2.4), scuttle, shadowtext, sf, SeuratObject,
        SpatialExperiment, STexampleData, testthat (>= 3.0.0), MASS
BugReports: https://github.com/YuLab-SMU/ggsc/issues
URL: https://github.com/YuLab-SMU/ggsc (devel),
        https://yulab-smu.top/ggsc/ (docs)
biocViews: DimensionReduction, GeneExpression, SingleCell, Software,
        Spatial, Transcriptomics,Visualization
VignetteBuilder: knitr
SystemRequirements: GNU make
ByteCompile: true
License: Artistic-2.0
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
Config/testthat/edition: 3
LinkingTo: Rcpp, RcppArmadillo, RcppParallel
Remotes: YuLab-SMU/SVP
Config/pak/sysreqs: cmake libglpk-dev make libicu-dev libpng-dev
        libuv1-dev libxml2-dev libssl-dev python3 zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-05-17 09:05:54 UTC
RemoteUrl: https://github.com/bioc/ggsc
RemoteRef: RELEASE_3_23
RemoteSha: d30c9bbc3174beb664c7ca019c8527621a23be07
NeedsCompilation: yes
Packaged: 2026-05-17 15:07:19 UTC; root
Author: Guangchuang Yu [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-6485-8781>),
  Shuangbin Xu [aut] (ORCID: <https://orcid.org/0000-0003-3513-5362>),
  Noriaki Sato [ctb]
Maintainer: Guangchuang Yu <guangchuangyu@gmail.com>
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-17 15:10:25 UTC; unix
