# Generated by roxygen2: do not edit by hand

S3method(ggplot_add,dim_geom_sub)
S3method(ggplot_add,dim_sub)
S3method(ggplot_add,sc_dim_geom_ellipse)
S3method(ggplot_add,sc_dim_geom_feature)
S3method(ggplot_add,sc_dim_geom_label)
S3method(ggplot_add,sc_geom_annot)
export("%<+%")
export(aes)
export(draw_key_bgpoint)
export(geom_bgpoint)
export(geom_scattermore2)
export(plot_lisa_feature)
export(sc_dim)
export(sc_dim_count)
export(sc_dim_geom_ellipse)
export(sc_dim_geom_feature)
export(sc_dim_geom_label)
export(sc_dim_geom_sub)
export(sc_dim_sub)
export(sc_dot)
export(sc_feature)
export(sc_geom_annot)
export(sc_geom_point)
export(sc_spatial)
export(sc_violin)
export(scale_bg_color_manual)
export(scale_bg_colour_discrete)
export(scale_bg_colour_manual)
export(theme)
exportMethods(sc_dim)
exportMethods(sc_dot)
exportMethods(sc_feature)
exportMethods(sc_spatial)
exportMethods(sc_violin)
importFrom(RColorBrewer,brewer.pal.info)
importFrom(Rcpp,evalCpp)
importFrom(Rcpp,sourceCpp)
importFrom(RcppParallel,RcppParallelLibs)
importFrom(Seurat,DefaultAssay)
importFrom(SingleCellExperiment,int_colData)
importFrom(SingleCellExperiment,int_metadata)
importFrom(SingleCellExperiment,reducedDimNames)
importFrom(SingleCellExperiment,reducedDims)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assayNames)
importFrom(SummarizedExperiment,colData)
importFrom(cli,cli_abort)
importFrom(ggfun,"%<+%")
importFrom(ggfun,get_aes_var)
importFrom(ggplot2,"%+replace%")
importFrom(ggplot2,ScaleDiscreteIdentity)
importFrom(ggplot2,aes)
importFrom(ggplot2,annotation_custom)
importFrom(ggplot2,coord_fixed)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,discrete_scale)
importFrom(ggplot2,draw_key_point)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,fill_alpha)
importFrom(ggplot2,geom_blank)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_violin)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggplot_add)
importFrom(ggplot2,ggplot_build)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,guide_colorbar)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,label_wrap_gen)
importFrom(ggplot2,labs)
importFrom(ggplot2,layer)
importFrom(ggplot2,layer_data)
importFrom(ggplot2,rel)
importFrom(ggplot2,scale_color_distiller)
importFrom(ggplot2,scale_color_gradient)
importFrom(ggplot2,scale_color_gradientn)
importFrom(ggplot2,scale_color_identity)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,stat_ellipse)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,translate_shape_string)
importFrom(ggplot2,waiver)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(grDevices,as.raster)
importFrom(grDevices,colorRampPalette)
importFrom(grid,gList)
importFrom(grid,gTree)
importFrom(grid,gpar)
importFrom(grid,grid.draw)
importFrom(grid,grobName)
importFrom(grid,grobTree)
importFrom(grid,is.unit)
importFrom(grid,pointsGrob)
importFrom(grid,rasterGrob)
importFrom(grid,unit)
importFrom(methods,as)
importFrom(methods,setMethod)
importFrom(rlang,.data)
importFrom(rlang,list2)
importFrom(rlang,sym)
importFrom(scales,alpha)
importFrom(scales,pal_identity)
importFrom(scattermore,geom_scattermore)
importFrom(scattermore,scattermore)
importFrom(stats,as.formula)
importFrom(stats,dist)
importFrom(stats,hclust)
importFrom(stats,setNames)
importFrom(tibble,as_tibble)
importFrom(tidydr,theme_dr)
importFrom(tidyr,pivot_longer)
importFrom(utils,getFromNamespace)
importFrom(utils,modifyList)
importFrom(yulab.utils,get_fun_from_pkg)
useDynLib(ggsc)
