#' @name reexports
#' @return Depending on the re-exported function
#' @keywords internal
NULL

##' @importFrom ggplot2 aes
##' @export
ggplot2::aes

##' @importFrom ggplot2 theme
##' @export
ggplot2::theme

##' @importFrom ggfun %<+%
##' @export
ggfun::`%<+%`
